//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef __GAMECLIENT_H_
#define __GAMECLIENT_H_

#ifndef __PLAYERPOSITION_H_
#define __PLAYERPOSITION_H_

#include <omnetpp.h>
#include <BasicModule.h>
#include <YMMOGMessage_m.h>
#include <vector>
#include <iostream>
#include "Movement.h"

class PlayerPosition {
	public:
		int playerId;
		int xpos;
		int ypos;
		double timeStamp;
		friend std::ostream& operator<<(std::ostream& Stream,const PlayerPosition player);
};

class GameClient :public BasicModule
{	
	public:
		virtual int numInitStages() const
		{
			return 6;
		}
		virtual void initialize(int stage);
		virtual void handleMessage(cMessage* msg);
		virtual void finish();
	protected:
		// parameters
		bool debugOutput;
	private:

		void sendMsg(cMessage* msg);
		
		int playerId;
		int xpos;
		int ypos;
		int gameSpaceSize; 
		
		// PlayerMap
		std::list<PlayerPosition> playerMap;
		std::list<PlayerPosition>::iterator pM;
		// Movement
		Movement* movement;
		// Timer
		cMessage* loginTimer;
		cMessage* moveTimer;
		cMessage* eventCreationTimer;

		// Delays
		double initialDelay;
		double eventCreationDelay;
		double moveDelay;
		// Statistics:
		int sentMessages;
		int receivedMessages;
		int sentEvents;
		int receivedEvents;
		int reachedWaypoints;
};

#endif
#endif
