//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.#endif
//

#include "Movement.h"

Movement::Movement(int max) {
	this->waypoint.x = 0;
	this->waypoint.y = 0;
	this->max = max;
	createWaypoint();
}

void Movement::createWaypoint() {
	waypoint.x = intuniform(0, max);
	waypoint.y = intuniform(0, max);
}

Point Movement::move(int xpos, int ypos) {
	/**
	Function is called by GameClient with actual Player Position
	GameClient expects a Point with the new Location and reached = true when something special like a Waypoint was reached.
	*/
	Point p;
	p.reached=false;
	p.x=xpos;
	p.y=ypos;
	
	// Move in x-direction of actual Waypoint:
	if( p.x - waypoint.x > 0 ) {
		p.x = xpos + intuniform(-100,30);
	}
	else {
		p.x = xpos + intuniform(-30,+100);
	}
	// Move in y-direction of actual Waypoint
	if( p.y - waypoint.y > 0 ) {
		p.y = ypos + intuniform(-100,30);
	}
	else {
		p.y = ypos + intuniform(-30,+100);
	}
	// Check Range / valid Output
	if(p.x < 0) p.x = 0;
	else if(p.x > max ) p.x = max;
	if(p.y < 0) p.y = 0;
	else if(p.y > max ) p.y = max;
	
	// Check if we are near of the actual waypoint:
	if ( abs( p.x - waypoint.x) < 50 ) p.x = waypoint.x;
	if ( abs( p.y - waypoint.y) < 50 ) p.y = waypoint.y;
	
	// Are we now at our Waypoint?
	if( p.x == waypoint.x && p.y == waypoint.y) {
		p.reached = true; // Yes let it GameClient know..
		createWaypoint(); // And create a new one
	}
	return p;
}


