//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file MovementGenerator.cc
 * @author Helge Backhaus
 */

#include "MovementGenerator.h"

MovementGenerator::MovementGenerator(double areaDimension, double speed, std::map<TransportAddress, Vector> *Neighbors)
{
    this->areaDimension = areaDimension;
    this->speed = speed;
    this->Neighbors = Neighbors;
}

void MovementGenerator::testBounds()
{
    if(position.getX() < -areaDimension) position.setX(-areaDimension);
    if(position.getX() > areaDimension) position.setX(areaDimension);
    if(position.getY() < -areaDimension) position.setY(-areaDimension);
    if(position.getY() > areaDimension) position.setY(areaDimension);
}

void MovementGenerator::flock()
{
    Vector vFlock;
    vFlock.setValue(0.0, 0.0);
    for(itNeighbors = Neighbors->begin(); itNeighbors != Neighbors->end(); ++itNeighbors) {
        if(position.distsqr(itNeighbors->second) < 4.0*speed*speed) {
            Vector temp;
            temp = itNeighbors->second - position;
            temp.normalize();
            vFlock += temp;
        }
    }
    vFlock.normalize();
    vFlock = vFlock * (speed/2.0);
    position -= vFlock;
}
