//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file greatGathering.cc
 * @author Helge Backhaus
 */

#include "greatGathering.h"

greatGathering::greatGathering(double areaDimension, double speed, std::map<TransportAddress, Vector> *Neighbors, Repository* repository)
               :MovementGenerator(areaDimension, speed, Neighbors)
{
    // get access to the repository
    this->repository = repository;

    position.setValue(uniform(-areaDimension, areaDimension), uniform(-areaDimension, areaDimension));
    flock();
    testBounds();

    if(repository->getPosition_size() == 0) {
        target.setValue(uniform(-areaDimension/2, areaDimension/2), uniform(-areaDimension/2, areaDimension/2));
        repository->setPosition_size(1);
        repository->setPosition(0, target);
    }
    else {
        Vector temp;
        target = repository->getPosition(0);
        temp.setValue(uniform(-20.0*speed, 20.0*speed), uniform(-20.0*speed, 20.0*speed));
        target += temp;
    }
    direction = target - position;
    direction.normalize();
}

void greatGathering::move()
{
    position += direction * speed;
    flock();
    testBounds();
    if(target.distsqr(position) < 4.0*speed*speed) {
        Vector temp;
        target = repository->getPosition(0);
        temp.setValue(uniform(-20.0*speed, 20.0*speed), uniform(-20.0*speed, 20.0*speed));
        target += temp;
    }
    direction = target - position;
    direction.normalize();
}
