//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file groupRoaming.cc
 * @author Helge Backhaus
 */

#include "groupRoaming.h"

groupRoaming::groupRoaming(double areaDimension, double speed, std::map<TransportAddress, Vector> *Neighbors, Repository* repository, int groupSize)
             :MovementGenerator(areaDimension, speed, Neighbors)
{
    // get access to the repository
    this->repository = repository;
    this->groupSize = groupSize;

    groupNr = repository->getCounter() / groupSize;

    if((repository->getCounter() % groupSize) == 0) {
        target.setValue(uniform(-areaDimension, areaDimension), uniform(-areaDimension, areaDimension));
        repository->setPosition_size(repository->getPosition_size() + 1);
        repository->setPosition(groupNr, target);
    }
    else {
        target = repository->getPosition(groupNr);
    }

    repository->setCounter(repository->getCounter() + 1);

    Vector temp;
    temp.setValue(uniform(-20.0*speed, 20.0*speed), uniform(-20.0*speed, 20.0*speed));
    position = target + temp;
    flock();
    testBounds();
    direction = target - position;
    direction.normalize();
}

void groupRoaming::move()
{
    if(groupNr <= repository->getCounter() / groupSize) { // not all members have spawned yet so just move a little
        flock();
        testBounds();
    }
    else {
        position += direction * speed;
        flock();
        testBounds();
        if(repository->getPosition(groupNr) != target) {
            target = repository->getPosition(groupNr);
        }
        if(target.distsqr(position) < 4.0*speed*speed) {
            target.setValue(uniform(-areaDimension, areaDimension), uniform(-areaDimension, areaDimension));
            repository->setPosition(groupNr, target);
        }
    }
    direction = target - position;
    direction.normalize();
}
