//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file myVector.cc
 * @author Helge Backhaus
 */

#include "myVector.h"

Vector::Vector()
{
    x = 0.0;
    y = 0.0;
}

void Vector::normalize()
{
    double temp;
    temp = sqrt((x*x + y*y));
    if(temp != 0.0) {
        x /= temp;
        y /= temp;
    }
}

void Vector::setValue(double x, double y)
{
    this->x = x;
    this->y = y;
}

double Vector::distsqr(const Vector& v)
{
    Vector temp;
    temp.setValue(x - v.x, y - v.y);
    return temp.x*temp.x + temp.y*temp.y;
}

Vector& Vector::operator=(const Vector& v)
{
    x = v.x;
    y = v.y;
    return *this;
}

Vector& Vector::operator+=(const Vector& v)
{
    x += v.x;
    y += v.y;
    return *this;
}

Vector& Vector::operator-=(const Vector& v)
{
    x -= v.x;
    y -= v.y;
    return *this;
}

Vector Vector::operator+(const Vector& v)
{
    Vector temp;
    temp.setValue(x + v.x, y + v.y);
    return temp;
}

Vector Vector::operator-(const Vector& v)
{
    Vector temp;
    temp.setValue(x - v.x, y - v.y);
    return temp;
}

Vector Vector::operator*(const double s)
{
    Vector temp;
    temp.setValue(x*s, y*s);
    return temp;
}

bool Vector::operator==(const Vector& v)
{
    if(this->x == v.x && this->y == v.y) return true;
    return false;
}

bool Vector::operator!=(const Vector& v)
{
    if(this->x == v.x && this->y == v.y) return false;
    return true;
}

std::ostream& operator<<(std::ostream& Stream, const Vector v)
{
    return Stream << "[" << v.x << ", " << v.y << "]";
}
