//
// Copyright (C) 2007 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file BinaryValue.cc
 * @author Ingmar Baumgart
 */

#include <iterator>

#include "BinaryValue.h"


using namespace std;

BinaryValue::BinaryValue(const char* s): vector<char>(strlen(s)) {
    copy(s, s+strlen(s), begin());  // Inherits vector<char>::begin()
}

BinaryValue::BinaryValue(const std::string& str) {
    *this = BinaryValue(str.c_str());
}

BinaryValue::BinaryValue(const std::vector<char>& v) : vector<char>(v) {};

// Allow output of vector<char> using normal notation
std::ostream& operator << (std::ostream& os, const BinaryValue& v) {
    copy(v.begin(), v.end(), ostream_iterator<char>(os, ""));
    return os;        // To allow (cout << a) << b;
}

// Automatically supply array (un)packing functions
template<typename T>
void doPacking(cCommBuffer *b, T *t, int n) {
    for (int i=0; i<n; i++)
        doPacking(b,t[i]);
}
template<typename T>
void doUnpacking(cCommBuffer *b, T *t, int n) {
    for (int i=0; i<n; i++)
        doUnpacking(b,t[i]);
}
inline void doPacking(cCommBuffer *, cPolymorphic&) {}
inline void doUnpacking(cCommBuffer *, cPolymorphic&) {}

#define DOPACKING(T,R) \
    inline void doPacking(cCommBuffer *b, T R a) {b->pack(a);}  \
    inline void doPacking(cCommBuffer *b, T *a, int n) {b->pack(a,n);}  \
    inline void doUnpacking(cCommBuffer *b, T& a) {b->unpack(a);}  \
    inline void doUnpacking(cCommBuffer *b, T *a, int n) {b->unpack(a,n);}
#define _
DOPACKING(char,_)
DOPACKING(unsigned char,_)
DOPACKING(bool,_)
DOPACKING(short,_)
DOPACKING(unsigned short,_)
DOPACKING(int,_)
DOPACKING(unsigned int,_)
DOPACKING(long,_)
DOPACKING(unsigned long,_)
DOPACKING(float,_)
DOPACKING(double,_)
DOPACKING(long double,_)
DOPACKING(char *,_)
DOPACKING(const char *,_)
DOPACKING(opp_string,&)
//DOPACKING(std::string,&)
#undef _
#undef DOPACKING

// predefined BinaryValue
const BinaryValue BinaryValue::UNSPECIFIED_VALUE;

void BinaryValue::netPack(cCommBuffer *b)
{
    doPacking(b,size());
    doPacking(b, (const char*)(&(*begin())), size());
}

void BinaryValue::netUnpack(cCommBuffer *b)
{
    int size;
    doUnpacking(b, size);
    resize(size);
    doUnpacking(b, (char*)(&(*begin())), size);
}
