//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file Geometry.h
 * @author Helge Backhaus
 */

#ifndef __GEOMETRY_H_
#define __GEOMETRY_H_

#include <math.h>
#include "VastDefs.h"

#define le 0
#define re 1

/// Geometry class
/**
 Provides basic line inter- / bisecting and processing functions, needed to build the voronoi and determine neighborhood relationships.
 */

class Geometry
{
    public:
        void initialize(double deltax, double deltay, Point center, Point old_pos, Point new_pos, double radius);
        void reset();
        void setDebug(bool debugOutput);
        Edge* bisect(Site *s1, Site *s2);
        Site* intersect(Halfedge *el1, Halfedge *el2);
        void endpoint(Edge *e, int lr, Site *s);
        void processEdge(Edge *e);
        double dist(Site *s, Site *t);

    protected:
        Site **SITEmemmap;
        Edge **EDGEmemmap;
        int SITEcount, EDGEcount;

        double deltax, deltay, sq_radius;
        Point center[3];
        bool debugOutput, doDiscovery;
        bool intersectCircleLine(Point start, Point dir, Point center, bool lowerBound, bool upperBound);
        bool intersectCircleSite(Site *s, Point center);
};

#endif
