//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file Heap.h
 * @author Helge Backhaus
 */

#ifndef __HEAP_H_
#define __HEAP_H_

#include "VastDefs.h"

/// HeapPQ class
/**
 Maintains a hash table in order to find halfedges in n*log(n) time.
 */

class HeapPQ
{
    public:
        HeapPQ();
        void PQinitialize(int sqrt_nsites, double ymin, double deltay);
        void PQreset();
        void PQinsert(Halfedge *he, Site *v, double offset);
        void PQdelete(Halfedge *he);
        int PQbucket(Halfedge *he);
        int PQempty();
        Point PQ_min();
        Halfedge* PQextractmin();

    protected:
        int PQcount, PQmin, PQhashsize;
        double ymin, deltay;
        Halfedge *PQhash;
};

#endif
