//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#include "LoadBalancingTree.h"
LoadBalancingTree::~LoadBalancingTree() {
	this->root->leafs.clear();
	this->root->childs.clear();
}
LoadBalancingTree::LoadBalancingTree() {
	this->root = new TreeNode();
	this->root->parent = NULL;
	this->lastTreeNode = this->root;
	this->size=0;
	this->lock = 0;
	actions=0;
	replacedIntermediateNodes = 0;
}

LoadBalancingTree::LoadBalancingTree(int subSpaceId, unsigned int maxPlayerLeafs, unsigned int treeWidth) {
	this->maxPlayerLeafs = maxPlayerLeafs;
	this->treeWidth = treeWidth;
	this->root = new TreeNode();
	this->root->parent = NULL;
	this->lastTreeNode = root;
	this->size = 0;
	this->lock = 0;
	this->subSpaceId = subSpaceId;
	actions=0;
	replacedIntermediateNodes = 0;
}


std::list<TreeMessage> LoadBalancingTree::addPlayer(int playerId, IPAddress playerIp) {
	actions++;
	std::list<TreeMessage> returnList;
	bool alreadyWaiting = false;
	for( iW = waitingNodes.begin() ; iW != waitingNodes.end(); ++iW ) {
		if( iW->playerId == playerId ) {
			alreadyWaiting = true;
		}
	}
	if ( searchPlayer(playerId).parent != NULL || alreadyWaiting ) {
		return returnList;
	}
	TreeMessage msg;
	if( ( (size > ( (usedIntermediateNodes.size() + readyIntermediateNodes.size()) * maxPlayerLeafs) + 0.5*maxPlayerLeafs) && lock == 0) || (size > (usedIntermediateNodes.size() + 2)*maxPlayerLeafs ) ) {
		lock = 1;
		TreeMessage newIntMsg;
		newIntMsg.type=2;
		returnList.push_back( newIntMsg);
	}
	msg.type = 0;
	msg.playerip = playerIp;
	if( freed.size() > 0 ) {
		TreeNode* foo = freed.front();
		msg.destip = foo->destip;
		if( foo->destip != (IPAddress) "127.0.0.1") returnList.push_back( msg);
		TreeNode blub = TreeNode( foo , playerId, playerIp);
		foo->leafs.push_back( blub );
		freed.pop_front();
		size++;
	}
	else if ( lastTreeNode->leafs.size() < maxPlayerLeafs) {
		msg.destip = lastTreeNode->destip;
		if( lastTreeNode->destip != (IPAddress) "127.0.0.1") returnList.push_back( msg);
		lastTreeNode->leafs.push_back( TreeNode( lastTreeNode, playerId, playerIp) );
		size++;
	}
	else {
		if( readyIntermediateNodes.size() > 0 ) {
			TreeNode* parent = lastTreeNode->parent;
			if( parent == NULL ) {
				parent = root;
			}
			
			IPAddress destip = readyIntermediateNodes.front();
			readyIntermediateNodes.pop_front();
			usedIntermediateNodes.push_back( destip );
			if( lastTreeNode != root && (parent->childs.size() < treeWidth) ) {
				//lastTreeNode->parent has enough space for another child
				parent->childs.push_back( TreeNode(parent, destip) ); 
				lastTreeNode = &( parent->childs.back()); // set lastTreeNode to the new child
				msg.destip = lastTreeNode->destip;
				if( lastTreeNode->destip != (IPAddress) "127.0.0.1") returnList.push_back( msg);
				lastTreeNode->leafs.push_back( TreeNode( lastTreeNode, playerId, playerIp)); // add player to new child
				size++;
			}
			else {
				lastTreeNode = searchLeftmostNode() ;
				lastTreeNode->childs.push_back( TreeNode(lastTreeNode, destip) );
				// Add |treeWidth| Players to the new Node. (specified in protocoll)
				TreeNode tmp;
				uint max = treeWidth;
				if( lastTreeNode->leafs.size() < treeWidth ) max= lastTreeNode->leafs.size();
				for( unsigned int i=0; i < max; ++i) {
					tmp = lastTreeNode->leafs.front();
					TreeMessage msg1;
					msg1.type = 1;
					msg1.destip = lastTreeNode->destip;
					msg1.playerip = tmp.destip;
					if( lastTreeNode->destip != (IPAddress) "127.0.0.1") returnList.push_back( msg1 );
					
					TreeMessage msg2;
					msg2.type = 0;
					msg2.destip = lastTreeNode->childs.back().destip; //=new Child
					msg2.playerip = tmp.destip;		
					returnList.push_back( msg2 );			 
					tmp.parent =  &(lastTreeNode->childs.back());
					lastTreeNode->childs.back().leafs.push_back(tmp);
					lastTreeNode->leafs.pop_front();
				}
				lastTreeNode = &( lastTreeNode->childs.back() );
				if( treeWidth == maxPlayerLeafs ) { // Not enough space for new Node (only if treeWidth=maxPlayerLeafs)
					waitingNodes.push_back( TreeNode( lastTreeNode, playerId, playerIp) );
				}
				else {
					msg.destip = lastTreeNode->destip;
					returnList.push_back( msg);
					lastTreeNode->leafs.push_back( TreeNode( lastTreeNode, playerId, playerIp)); //add Player to the new Node
					size++;
				}
					
			}
		}
		else {
			waitingNodes.push_back( TreeNode( lastTreeNode, playerId, playerIp) );
		}
	}
	return returnList;
}

std::list<TreeMessage> LoadBalancingTree::removePlayer(int playerId) {

	actions++;
	std::list<TreeMessage> returnList;
	NodePos playerPos = searchPlayer(playerId);
	if ( playerPos.parent != NULL) {
		size--;
		TreeMessage msg;
		msg.type = 1;
		msg.destip = playerPos.parent->destip;
		msg.playerip = playerPos.it->destip;
		returnList.push_back( msg );
		playerPos.parent->leafs.erase(playerPos.it);
		freed.push_back( playerPos.parent );
		if( freed.size() > maxPlayerLeafs + 1 ) {
			lastTreeNode = searchLastNode();
			// We have to delete one Intermediate TreeNode
			TreeNode tmp;
			while( lastTreeNode->leafs.size() > 0 && freed.size() > 0 ) {
				if( &*freed.front() != &*lastTreeNode ) {
					tmp = lastTreeNode->leafs.front();
					TreeMessage msg1;
					msg1.type = 0;
					msg1.playerip = tmp.destip;
					msg1.destip = freed.front()->destip;
					tmp.parent = freed.front();
					freed.front()->leafs.push_back( tmp );
					returnList.push_back(msg1);
					TreeMessage msg2;
					msg2.type = 1;
					msg2.playerip = tmp.destip;
					msg2.destip = lastTreeNode->destip;
					returnList.push_back(msg2);
					lastTreeNode->leafs.pop_front();
				}
				freed.pop_front();
			}
			//Check if we have an entry in free by ourself
			std::list<TreeNode*>::iterator f = freed.begin();
			while( f != freed.end() ) {
				if( &**f == &*lastTreeNode ) {
					f = freed.erase(f);
				}
				else { ++f; }
			}
			//check if we're in replace 
			f = inReplace.begin();
			while( f != inReplace.end() ) {
				if( &**f == &*lastTreeNode ) {
					f = inReplace.erase(f);
				}
				else { ++f; }
			}
			//Remove us from usedIntermediateNodes
			std::list<IPAddress>::iterator ip = usedIntermediateNodes.begin();
			while( ip != usedIntermediateNodes.end() ) {
				if( *ip == lastTreeNode->destip ) {
					usedIntermediateNodes.erase(ip);
					break;
				}
				else { ++ip; }
			}
			TreeMessage msg3;
			msg3.type=3;
			msg3.destip = lastTreeNode->destip;
			returnList.push_back( msg3 );
			
			TreeNode* toRemoved = lastTreeNode;
			lastTreeNode = lastTreeNode->parent;
			if( lastTreeNode == NULL ) lastTreeNode = root;
			for(lastTreeNode->ic = lastTreeNode->childs.begin() ; lastTreeNode->ic != lastTreeNode->childs.end(); ++(lastTreeNode->ic) ) {
				if( &*(lastTreeNode->ic) == toRemoved ) {
					lastTreeNode->childs.erase(lastTreeNode->ic);
					break;
				}
			}
				
			if( lastTreeNode->childs.size() > 0 ) {
				lastTreeNode = &( lastTreeNode->childs.back() );
			}
		}
	}
	else {
		bool found = false;
		for( iW = waitingNodes.begin(); iW != waitingNodes.end(); ++iW ) {
			if( iW->playerId == playerId ) {
				waitingNodes.erase(iW);
				found = true;
				break;
			}
		}
	}
	return returnList;
}

std::list<TreeMessage> LoadBalancingTree::removePlayer(IPAddress playerIp) {
	actions++;
	std::list<TreeMessage> returnList;	
	NodePos playerPos = searchPlayer(playerIp);
	if( playerPos.parent != NULL ) {
		returnList = removePlayer( playerPos.it->playerId );
		std::list<TreeNode*> toReplace = searchNodes( playerIp);
		if( toReplace.size() > 0 ) {
			while( toReplace.size() > 0 ) {
				inReplace.push_back ( toReplace.front() );
				toReplace.pop_front();
				TreeMessage msg;
				msg.type = 2;
				returnList.push_back( msg );	
			}
		}
	}
	return returnList;
}

std::list<IPAddress> LoadBalancingTree::exchangeNode(IPAddress oldNodeIp, IPAddress newNodeIp) {
	std::list<IPAddress> returnList;	
	std::list<TreeNode*> toReplace = searchNodes(oldNodeIp);
	while( toReplace.size() > 0 ) {
		inReplace.push_front( toReplace.front() );
		std::list<TreeNode>::iterator i;
		for( i = toReplace.front()->childs.begin() ; i != toReplace.front()->childs.end(); ++i) {
			returnList.push_back( i->destip);
		}
		for( i = toReplace.front()->leafs.begin() ; i != toReplace.front()->leafs.end(); ++i) {
			returnList.push_back( i->destip);
		}
		replace(newNodeIp);
		toReplace.pop_front(); //missing messages

	}
	return returnList;	
}

std::list<IPAddress> LoadBalancingTree::getList() {
	std::list<IPAddress> ipList; 
	for(root->ic = root->childs.begin(); root->ic != root->childs.end(); ++root->ic) {
		ipList.push_back(root->ic->destip);
	}
	for(root->il = root->leafs.begin(); root->il != root->leafs.end() ; ++root->il) {
		ipList.push_back(root->il->destip);
	}
	for( iR = inReplace.begin(); iR != inReplace.end(); ++iR) {
		TreeNode* visit = *iR;
		for( visit->ic = visit->childs.begin(); visit->ic != visit->childs.end(); ++(visit->ic) ) {
			ipList.push_back( visit->ic->destip);
		}
		for( visit->il = visit->leafs.begin(); visit->il != visit->leafs.end(); ++(visit->il) ) {
			ipList.push_back( visit->il->destip);
		}
	}
	
	for( iW = waitingNodes.begin(); iW != waitingNodes.end(); ++iW) {
		ipList.push_back( iW->destip );
	}
	return ipList;
}


std::list<PlayerNode> LoadBalancingTree::getAllPlayers() {
	std::list<PlayerNode> returnList;
	
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	TreeNode* visit;
	std::list<TreeNode>::iterator il;
	std::list<TreeNode>::iterator ic;

	while ( queue.size() > 0 ) {	
		visit = queue.front();
		for( il = visit->leafs.begin(); il != visit->leafs.end(); ++il) {
			returnList.push_back( PlayerNode( il->playerId, il->destip, 0, 0) );
		}
		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
		}
		queue.pop_front();
	}
	return returnList;
}

std::list<IPAddress> LoadBalancingTree::getAllNodes() {
	std::list<IPAddress> returnList;
	
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	TreeNode* visit;
	std::list<TreeNode>::iterator il;
	std::list<TreeNode>::iterator ic;

	while ( queue.size() > 0 ) {	
		visit = queue.front();
		for( il = visit->leafs.begin(); il != visit->leafs.end(); ++il) {
			returnList.push_back( il->destip);
		}
		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
			returnList.push_back(ic->destip);
		}
		queue.pop_front();
	}
	return returnList;
}
std::list<IPAddress> LoadBalancingTree::getAllIntNodes() {
	std::list<IPAddress> returnList;	
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	TreeNode* visit;
	std::list<TreeNode>::iterator ic;

	while ( queue.size() > 0 ) {	
		visit = queue.front();
		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
			returnList.push_back(ic->destip);
		}
		queue.pop_front();
	}
	return returnList;
}

NodePos LoadBalancingTree::searchPlayer(int playerId) {
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	TreeNode* visit;
	std::list<TreeNode>::iterator il;
	std::list<TreeNode>::iterator ic;

	while ( queue.size() > 0 ) {		
		visit = queue.front();
		for( il = visit->leafs.begin(); il != visit->leafs.end(); ++il) {
			if( il->playerId == playerId ) {
				NodePos playerPos;
				playerPos.parent = il->parent;
				playerPos.it = il;
				return playerPos;
			}
		}
		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
		}
		queue.pop_front();
	}
	NodePos playerPos;
	playerPos.parent = NULL;
	return playerPos;
}



NodePos LoadBalancingTree::searchPlayer(IPAddress playerIp) {
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	TreeNode* visit;
	std::list<TreeNode>::iterator il;
	std::list<TreeNode>::iterator ic;

	while ( queue.size() > 0 ) {
		visit = queue.front();
		for( il = visit->leafs.begin(); il != visit->leafs.end(); ++il) {
			if( il->destip == playerIp ) {
				NodePos playerPos;
				playerPos.parent = il->parent;
				playerPos.it = il;
				return playerPos;
			}
		}
		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
		}
		queue.pop_front();
	}
	NodePos playerPos;
	playerPos.parent = NULL;
	return playerPos;
}

TreeNode* LoadBalancingTree::searchLeftmostNode() {
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	TreeNode* visit;
	while ( queue.size() > 0 ) {
		visit = queue.front();
		if( visit->childs.size() < treeWidth ) {
			return visit;
		}
		for( visit->ic = visit->childs.begin(); visit->ic != visit->childs.end(); ++visit->ic) {
			queue.push_back( &(*(visit->ic)) );
		}
		queue.pop_front();
	}
	return root;
}

TreeNode* LoadBalancingTree::searchLastNode() {
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	TreeNode* visit = root;
	while ( queue.size() > 0 ) {
		visit = queue.front();
		for( visit->ic = visit->childs.begin(); visit->ic != visit->childs.end(); ++visit->ic) {
			queue.push_back( &(*(visit->ic)) );
		}
		queue.pop_front();
	}
	return visit;
}


std::list<TreeNode*> LoadBalancingTree::searchNodes(IPAddress destip) {
	std::list<TreeNode*> returnList;
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	std::list<TreeNode>::iterator ic;
	TreeNode* visit;
	while( queue.size() > 0 ) {
		visit = queue.front();
		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			if( ic->destip == destip ) {
				returnList.push_back( &(*ic) );
			}
			queue.push_back( &(*ic) );
		}
		queue.pop_front();
	}
	return returnList;
}

std::list<TreeMessage> LoadBalancingTree::replace(IPAddress newNodeIp) {
	std::list<TreeMessage> returnList;
	if( inReplace.size() > 0 ) {
		TreeNode* node = inReplace.front();
		node->destip = newNodeIp;
		for(node->il = node->leafs.begin(); node->il != node->leafs.end(); ++(node->il) ) {
			TreeMessage msg1;
			msg1.type = 0;
			msg1.playerip = node->il->destip;
			msg1.destip = newNodeIp;
			returnList.push_back(msg1);
		}
		replacedIntermediateNodes++;
		inReplace.pop_front();
	}
	return returnList;
}

std::list<TreeMessage> LoadBalancingTree::addIntermediateNode(IPAddress destip) {
	std::list<TreeMessage> returnList;
	lock = 0;
	if( inReplace.size() > 0 ) {
		returnList = replace(destip);
	}
	else {
		readyIntermediateNodes.push_back( destip );
		uint i = 0;
		while( waitingNodes.size() > 0 && i < maxPlayerLeafs ) {
			TreeNode tmp2 = waitingNodes.front();
			waitingNodes.pop_front();
			std::list<TreeMessage> tmp = this->addPlayer( tmp2.playerId, tmp2.destip);
			while( tmp.size() > 0 ) {
				returnList.push_back( tmp.front() );
				tmp.pop_front();
			}
			i++;
		}
	}
	return returnList;

}

std::list<TreeNode> LoadBalancingTree::serialize() {
	std::list<TreeNode> returnVector;
	std::list<TreeNode*> queue;
	queue.push_back( root );
	std::list<TreeNode>::iterator ic;
	std::list<TreeNode>::iterator il;
	std::list<TreeNode*>::iterator ir;
	std::list<IPAddress>::iterator ip;
	TreeNode* visit;
	while ( queue.size() > 0 ) {
		visit = queue.front();
		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
			returnVector.push_back(TreeNode(-1, ic->destip));
		}
		for( il = visit->leafs.begin(); il != visit->leafs.end(); ++il) {
			returnVector.push_back( TreeNode(il->playerId, il->destip) );
		}
		returnVector.push_back( TreeNode(-5, (IPAddress) "127.0.0.1" ) );
		queue.pop_front();
	}
	returnVector.push_back( TreeNode(-6, (IPAddress) "127.0.0.1" ) );
	for(ip = readyIntermediateNodes.begin() ; ip != readyIntermediateNodes.end() ; ++ip) {
		returnVector.push_back( TreeNode( -3, (*ip) ) );
	}
	returnVector.push_back( TreeNode(-7, (IPAddress) "127.0.0.1" ) );
	for(il = waitingNodes.begin(); il != waitingNodes.end(); ++il ) {
		returnVector.push_back( TreeNode(il->playerId, il->destip) );
	}
	returnVector.push_back( TreeNode(-8, (IPAddress) "127.0.0.1" ) );

	for(ir = inReplace.begin(); ir != inReplace.end(); ++ir) {
		returnVector.push_back( TreeNode( -2 , (*ir)->destip ) );
	}
	returnVector.push_back( TreeNode( -10 - replacedIntermediateNodes, (IPAddress) "127.0.0.1") );
	return returnVector;
}

void LoadBalancingTree::rebuildTree(std::list<TreeNode> source) {
	std::list<TreeNode>::iterator i= source.begin();
	this->root->parent = NULL;
	int state=0;
	std::list<TreeNode*> queue;
	std::list<TreeNode*>::iterator q;
	queue.push_back( root );
	for( i = source.begin(); i != source.end(); ++i ) {
		if( state == 0) {
			if( i->playerId == -1 ) {
				// create a new child in queue.front
				(queue.front())->childs.push_back( TreeNode( queue.front(), i->playerId, i->destip ) );
				queue.push_back( &((queue.front())->childs.back()) );
			}
			else if( i->playerId == -5 ) {
				// go to next child
				uint k=0;
				// has actual node some free leaf slots?
				for(k=queue.front()->leafs.size(); k < maxPlayerLeafs - treeWidth; ++k) {
					freed.push_back(queue.front());
				}
				queue.pop_front();
			}
			else if ( i->playerId > 0) {
				(queue.front())->leafs.push_back( TreeNode( queue.front(), i->playerId, i->destip ) );
			}
			else if ( i->playerId == -6 ) {
				state = 1;
			}
		}
		else if ( state == 1 ) {
			if ( i->playerId == -3 ) {
				readyIntermediateNodes.push_back( i->destip );
			}
			else if ( i->playerId == -7 ) {
				state = 2;
			}
		}
		else if ( state == 2) {
			if ( i->playerId > 0 ) {
				waitingNodes.push_back( TreeNode(queue.front(), i->playerId, i->destip ) );
			}
			else if( i->playerId == -8 ) {
				state = 3;
			}
		}
		else if ( state == 3) {
			if ( i->playerId == -2 ) {
				std::list<TreeNode*> pos = searchNodes( i->destip );
				std::list<TreeNode*>::iterator p;
				for( p = pos.begin(); p != pos.end(); ++p ) {
					if( (*p)->playerId == -1 ) {
						inReplace.push_back( *p );
					}
				}
			}
			else if( i->playerId < -10) {
				replacedIntermediateNodes = abs(i->playerId) - 10;
			}
		}
	}
	lastTreeNode = searchLeftmostNode();
}
void LoadBalancingTree::printTree() {
	std::cout << root << " " << lastTreeNode << " " << " " << waitingNodes.size() << " " << inReplace.size() << " " <<freed.size() << "\n";	
	std::list<TreeNode*> queue;
	queue.push_back( root );
	std::list<TreeNode>::iterator ic;
	TreeNode* visit;
	int num= 0;
	int layer=0;
	while ( queue.size() > 0 ) {
		visit = queue.front();
		std::cout << "NodeNum: " << num << " ChildSize: " << visit->childs.size() << " LeafSize: " << visit->leafs.size() << " NodeIp: " << visit->destip << " | ";
		if( num % treeWidth == 0) std::cout << " ### ";
		if( num == pow(treeWidth, layer)  || num == 0 ) {
			std::cout << "\n\n";
			layer++;
		}
		num++;

		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
		}
		queue.pop_front();
	}		
}

std::ostream& operator<<(std::ostream& Stream, LoadBalancingTree t) {
	std::ostream& output = Stream << "Size: " << t.size << "readyIntermediateNodes " << t.readyIntermediateNodes.size() << "InReplace: " << t.inReplace.size() << " Freed: " << t.freed.size() << "\n\n";
	
	std::list<TreeNode*> queue;
	queue.push_back( t.root );
	std::list<TreeNode>::iterator ic;
	TreeNode* visit;
	int num= 0;
	int layer=0;
	while ( queue.size() > 0 ) {
		visit = queue.front();
		output << "NodeNum: " << num << " ChildSize: " << visit->childs.size() << " LeafSize: " << visit->leafs.size() << " NodeIp: " << visit->destip << " | ";
		if( num % t.treeWidth == 0) output << " ### ";
		if( num == pow(t.treeWidth, layer)  || num == 0 ) {
			output << "\n\n";
			layer++;
		}
		num++;

		for( ic = visit->childs.begin(); ic != visit->childs.end(); ++ic) {
			queue.push_back( &(*ic) );
		}
		queue.pop_front();
	}		
	return output;
}
