//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef __LOADBALANCINGTREE_H_
#define __LOADBALANCINGTREE_H_

#include <list>
#include "IPAddress.h"
#include "TreeNode.h"
#include "PlayerNode.h"

struct NodePos {
	TreeNode* parent;
	std::list<TreeNode>::iterator it;
};
struct TreeMessage {
	int type;
	IPAddress playerip;
	IPAddress destip;
};

class LoadBalancingTree {
	public:
		~LoadBalancingTree();
		LoadBalancingTree();
		LoadBalancingTree(int subSpaceId, unsigned int maxPlayerLeafs, unsigned int treeWidth);
		std::list<TreeMessage> addPlayer(int playerId,  IPAddress playerIp);
		std::list<TreeMessage> removePlayer(int playerId);
		std::list<TreeMessage> removePlayer(IPAddress playerIp);
		std::list<IPAddress> getList();
		std::list<PlayerNode> getAllPlayers();
		std::list<IPAddress> getAllNodes();
		std::list<IPAddress> getAllIntNodes();
		std::list<TreeMessage> addIntermediateNode(IPAddress destip);
		std::list<IPAddress> exchangeNode(IPAddress oldNodeIp, IPAddress newNodeIp);
		std::list<TreeMessage> replace(IPAddress newNodeIp);
		unsigned int size;
		friend std::ostream& operator<<(std::ostream& Stream, LoadBalancingTree t);
		std::list<IPAddress> readyIntermediateNodes;
		std::list<IPAddress> usedIntermediateNodes;
		std::list<TreeNode> serialize();
		void rebuildTree(std::list<TreeNode> source);
		void printTree();
		int replacedIntermediateNodes;	
	protected:
		TreeNode* root;
		TreeNode* lastTreeNode;
		NodePos searchPlayer(int playerId);
		NodePos searchPlayer(IPAddress playerIp);
		//std::list<NodePos> searchNodes(IPAddress playerIp);
		std::list<TreeNode*> searchNodes(IPAddress playerIp);
		std::list<TreeNode> waitingNodes;
		std::list<TreeNode>::iterator iW;
		TreeNode* searchLeftmostNode();
		TreeNode* searchLastNode();
		std::list<TreeNode*> freed;
		std::list<TreeNode*>::iterator f;
	//	std::list<NodePos> inReplace;
	//	std::list<NodePos>::iterator iR;
		std::list<TreeNode*> inReplace;
		std::list<TreeNode*>::iterator iR;
			
		std::list<IPAddress>::iterator rI;
		unsigned int maxPlayerLeafs;
		unsigned int treeWidth;
		int lock;
		int subSpaceId;
		int actions;	
};

#endif
