//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
/**
 * @file "LobbyServer.h"
 * @author "Holger Miller"
*/

#ifndef __LOBBYSERVER_H_
#define __LOBBYSERVER_H_

#include "YMMOGBase.h"
#include <vector>
#include <list>
#include <omnetpp.h>
#include <BootstrapOracle.h>
#include <NodeHandle.h>
#include "PlayerTable.h"
#include "SubSpaceTable.h"
#include "YMMOGMessage_m.h"
#include "NodeLogEntry.h"

/**
 * Main Class for LobbyServer
 *
 * Class for SimpleModule LobbyServer.ned . Should be part of Compound Module CentralLobbyServer.ned
 * This class handels all messages. It holds a Pointer to one PlayerTable and one SubSpaceTable.
 * It calls BootstrapOracle->registerPeer on initialize(). LobbyServer should be the only entry in the BootstrapOracle.
 * It also collects statistics about usage on finish().
 *
 * @author "Holger Miller ( holger.miller@travian.org )"
*/
class LobbyServer :public YMMOGBase
{
	public:
		~LobbyServer();
		virtual int numInitStages() const {return 6;}
		// OMNeT++
		    /**
		     * initializes base-class-attributes
		     *
		     * At Stage 5 it reads parameters from .ini , creates playerOnlineTable and subSpaceTable setup Watches etc and calls BootstrapOracle->registerPeer
		     *
		     * @param stage the init stage
		     */
		void initialize();
		void finish();
	protected:
	private:
		//module references
		PlayerTable* playerOnlineTable;
		SubSpaceTable* subSpaceTable;
	
		
		void informNeighbors(SubSpace* subSpace);
		bool createResponsibleNode(int id);
		
		
		int login(int id, IPAddress ip, int Bandwith, int Cpu);
		bool logout(int id);
		
		
		// Parameters
		int gameSpaceSize;
		int subSpaceSize;
		
		//Login
		void addSubSpaceToOpenLogins(int subSpaceId);
		// RequestIpMessage
		void sendWaitingRequests(int subSpaceId, int nodeType);		
		//Ping
		void sendPing(PingMessage* openPing);
		
		void processPingTimer();
		void processStabilizeTimer();
		void processSelfTimers(cMessage* msg);

		/** Message Handling **/
		void processLogin(LoginMessage* loginMsg);		
		void processLogout(LogoutMessage* logoutMsg);
		void processCreate(CreateNodeMessage* createMsg);
		void processRequestIp(RequestIpMessage* requestMsg);
		void processRequestNode(RequestNodeMessage* requestMsg);
		void processUpdateRespIp(UpdateRespIpMessage* updateMsg);		
		void processReplacement(ReplacementMessage* replaceMsg);
		void processPing(PingMessage* pingMsg);
		void processStabilize(StabilizeMessage* smsg);
		
		int maxFailedPings;
		int completedLogins;
		bool thereIsANotReadyCreate;
		
};

#endif
