//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#include <vector>
#include <set>
#include "PlayerNode.h"
#include "GameState.h"
#include "Event.h"
#include "LoadBalancingTree.h"
#include "SubSpace.h"
#include <iostream>

class ResponsibleNodeData
 {
	public:
		// OMNeT++
		~ResponsibleNodeData();
		//ResponsibleNodeData(int id, int c, int k);
		ResponsibleNodeData(SubSpace subSpace , int maxPlayerLeafs , int treeWidth, double time);
		ResponsibleNodeData(SubSpace subSpace , int maxPlayerLeafs , int treeWidth, double time, std::list<TreeNode> tree);
		std::list<TreeMessage> subscribe(PlayerNode player);
		std::list<TreeMessage> unsubscribe(int playerId);
		std::list<TreeMessage> unsubscribe(IPAddress playerIp);
		std::list<IPAddress> getList();
		std::list<TreeNode> getPlayerList();
		std::list<IPAddress> getNodeList(); 
		
		std::list<TreeMessage> addIntermediateNode(IPAddress intermediateIp);
		std::list<IPAddress> replaceIntermediateNode(IPAddress oldip, IPAddress newip);
		
		void addEvent(Event event);
		void setNeighbor(int direction, IPAddress resp_ip);
		int getEventListSize();
		std::vector<Event> endTimeSlot(double now); 
		int getPlayerListSize();
		std::list<IPAddress>getAllIntNodes();
		
		SubSpace getSubSpace();
		void setSubSpace(SubSpace subSpace);
		void setBackupIp(IPAddress backupIp);
		IPAddress getBackupIp();
		int getId();
		double getDroppedEvents();
		int getReplacedIntermediateNodeCount();
		void setMaxPlayerLeafs(int maxPlayerLeafs);
		void setTreeWidth( int treeWidth);
		friend std::ostream& operator<<(std::ostream& Stream, ResponsibleNodeData r);

	protected:
		std::list<Event> eventlist;
		std::list<Event>::iterator it;
		std::list<IPAddress> readyIntermediateNodes;
		std::list<IPAddress>::iterator rdyIp;
		std::list<PlayerNode> tmpList;
		std::list<PlayerNode>::iterator tmpListIt;
		
		GameState gamestate;
		int treeWidth;
		int maxPlayerLeafs;
		double last_timeslot;
		int dropped_events;
		int eventCount;
		LoadBalancingTree* tree;
		SubSpace subSpace;
	private:
		PlayerNode* getPlayerbyId(int id);

};

