//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef __SUBSPACETABLE_H_
#define __SUBSPACETABLE_H_

#include <vector>
#include <list>
#include <BasicModule.h>
#include <omnetpp.h>
#include <iostream>
#include "SubSpace.h"

class SubSpaceTable :public BasicModule
{
	public:
		~SubSpaceTable();
		virtual int numInitStages() const {return 6;}
		// OMNeT++
		virtual void initialize(int stage);
		virtual void finish();
		virtual void handleMessage(cMessage* msg);
		void setResponsibleNode(int id, IPAddress resp_ip);
		void setBackupNode(int id, IPAddress backup_ip);
		void createTable(int size);
		SubSpace* getSubSpace(unsigned int id);
		int getSize();
		std::vector<SubSpace*> getAllNeighbors(int id);
		void toString();
		void addNode(int subSpaceId, IPAddress nodeIp);
		void removeNode(int subSpaceId, IPAddress nodeIp);
		bool isInNodeList(int subSpaceId, IPAddress nodeIp);
		std::vector< std::list<IPAddress> > nodeList;
		void record(int subSpaceId, int logType, double value);
	protected:
	private:
		void processCollectStats();
		int size;
		std::vector<SubSpace*> subSpaceTable;
		
		/*
		std::vector<cOutVector*> maxEventDelayLog;
		std::vector<cOutVector*> avgEventDelayLog;
		std::vector<cOutVector*> minEventDelayLog;
		*/
		std::vector<cOutVector*> intNodesCountLog;
		std::vector<cOutVector*> playerOnlineLog;
		//std::vector<cOutVector*> replacedNodesLog;
		std::vector<int>replacedNodesLog;
		//std::vector<cOutVector*> droppedEventsLog;
		/*std::vector<cOutVector*> maxCommunicationLog;
		std::vector<cOutVector*> avgCommunicationLog;
		std::vector<cOutVector*> minCommunicationLog;
		std::vector<cStdDev> communicationStats;
		*/
		cOutVector sessionTime;
		//cStdDev eventDelay;
		//cStdDev treeDelay;
		cOutVector* treeLog;
		//cOutVector eventLog;
		cOutVector droppedEventsLog;
		cOutVector* respLogUp;
		cOutVector* respLogDown;
		cOutVector* backupLogUp;
		cOutVector* backupLogDown;
		cOutVector* intLogUp;
		cOutVector* intLogDown;
		
		cOutVector* eventOut;
		unsigned long eventLog[1000];
		unsigned long eventCount;
		
		/*
		cOutVector maxEventDelay;
		cOutVector minEventDelay;
		cOutVector avgEventDelay;
		
		cOutVector maxTreeDelay;
		cOutVector minTreeDelay;
		cOutVector avgTreeDelay;
		*/
		cMessage* collectStatsTimer;
		double collectStatsDelay;
};

#endif
