//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file udpoutscheduler.cc
 * @author Stephan Krause
 */

#include "udpoutscheduler.h"

Register_Class(UdpOutScheduler);

UdpOutScheduler::~UdpOutScheduler()
{
    if (app_fd >= 0) {
	close(app_fd);
    }
    if (additional_fd >= 0) {
	close(additional_fd);
    }

    delete dev;
}


int UdpOutScheduler::initializeNetwork()
{
    dev = new char[IFNAMSIZ];

    // get app port (0 if external app is not used)
    int appPort = ev.config()->getAsInt("ExternalApp", "app-port");

    // Initialize TCP socket for App communication if desired
    if ( appPort > 0 ) {

	struct sockaddr_in server;
	int sock;

	// Waiting for a TCP connection
	// WARNING: Will only accept exactly ONE app connecting to the socket
	sock = socket( AF_INET, SOCK_STREAM, 0 );
	if (sock < 0) {
	    opp_error("Error creating socket");
	    return -1;
	}

	int on = 1;
	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));

	memset( &server, 0, sizeof (server));
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = htonl( INADDR_ANY );
	server.sin_port = htons( appPort );

	if(bind( sock, (struct sockaddr*)&server, sizeof( server)) < 0) {
	    opp_error("Error binding to app socket");
	    return -1;
	}
	if( listen( sock, 5 ) == -1 ) {
	    opp_error("Error listening on app socket");
	    return -1;
	}
	// Set additional_fd so we will be called if data
	// (i.e. connection requests) is available at sock
	additional_fd = sock;
    }

    // Open UDP port
    if (netw_fd > 0) {
	// Port is already open, reuse it...
	return 0;
    }

    sockaddr_in addr;
    netw_fd = socket( AF_INET, SOCK_DGRAM, 0 );
    memset( &addr, 0, sizeof(addr) );
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl( INADDR_ANY );
    addr.sin_port = htons(
	    simulation.moduleByPath("SingleHostNetwork.singleHost.overlay")->par("localPort").longValue()
    );
    if(bind( netw_fd, (sockaddr*)&addr, sizeof(addr)) < 0) {
	opp_error("Error binding to UDP socket");
	return -1;
    }


    return 0;
}

void UdpOutScheduler::additionalFD() {
    int new_sock = accept( additional_fd, 0, 0 );
    if (app_fd >= 0) {
	// We already have a connection to an application
	// "reject" connection
	close(new_sock);
	ev << "Rejecting new app connection (FD: " << new_sock << ")\n";
    } else {
	app_fd = new_sock;
	ev << "Accepting new app connection (FD: " << app_fd << ")\n";
	if (app_fd < 0) {
	    opp_warning("Error connecting to remote app");
	    app_fd = -1;
	}
    }

}

